{ Copyright (C) 2017 H Page-Clark

  This library is free software; you can redistribute it and/or modify it
  under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version with the following modification:

  As a special exception, the copyright holders of this library give you
  permission to link this library with independent modules to produce an
  executable, regardless of the license terms of these independent modules,and
  to copy and distribute the resulting executable under terms of your choice,
  provided that you also meet, for each linked independent module, the terms
  and conditions of the license of that module. An independent module is a
  module which is not derived from or based on this library. If you modify
  this library, you may extend this exception to your version of the library,
  but you are not obligated to do so. If you do not wish to do so, delete this
  exception statement from your version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, write to the Free Software Foundation,
  Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1335, USA.  }

unit uVMenu;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls, Graphics, Controls, Types, LCLType, LCLIntf;

const
  ItemHeight = 30;     // change these to suit
  MenuWidth = 200;
  CaptionOffset = 25;

type

  { TLabelEx }

  TLabelEx = class(TCustomLabel)
  private
    BackgroundColor: TColor;
    HiliteColor: TColor;
    Ht: Integer;
    TextHeight: Integer;
    Wd: Integer;
  protected
    procedure MouseEnter; override;
    procedure MouseLeave; override;
  public
    constructor Create(anOwnerParent: TCustomControl;
                       const aCaption: string;
                       aFontColor, aHiliteColor, aBackgroundColor: TColor;
                       aWidth, aHeight, aTextHeight: Integer); reintroduce;
    procedure Paint; override;
  end;

  TLabelExArray = array of TLabelEx;

  { TVerticalMenu }

  TVerticalMenu = class(TCustomControl)
  private
    HiliteColor: TColor;
    LabelArray: TLabelExArray;
    function GetCaptions(index: Integer): string;
    function GetCount: Integer;
    function GetOnClick(index: Integer): TNotifyEvent;
    function GetTextHeight: Integer;
    function IndexIsOK(anIndex: Integer): Boolean;
    procedure SetCaptions(index: Integer; AValue: string);
    procedure SetOnClick(index: Integer; AValue: TNotifyEvent);
  public
    constructor Create(anOwnerParent: TWinControl;
                       const aCaptionsCommaText: string;
                       aBackgroundColor, aFontColor, aHiliteColor: TColor); reintroduce;
    property Captions[index: Integer]: string read GetCaptions write SetCaptions;
    property Count: Integer read GetCount;
    property OnClick[index: Integer]: TNotifyEvent read GetOnClick write SetOnClick;
  end;

implementation

{ TVerticalMenu }

constructor TVerticalMenu.Create(anOwnerParent: TWinControl;
  const aCaptionsCommaText: string; aBackgroundColor, aFontColor,
  aHiliteColor: TColor);
var
  sl: TStringList;
  i, highIdx, th: integer;
begin
  Assert(Assigned(anOwnerParent),'TVerticalMenu.Create: owner is nil');
  Assert(aCaptionsCommaText<>'','TVerticalMenu.Create: no menu captions exist');
  inherited Create(anOwnerParent);
  Color:=aBackgroundColor;
  HiliteColor:=aHiliteColor;
  Parent:=anOwnerParent;
  th:=GetTextHeight;
  sl:=TStringList.Create;
  try
    sl.CommaText:=aCaptionsCommaText;
    highIdx:=sl.Count-1;
    for i:=0 to highIdx do begin
      SetLength(LabelArray, i+1);
      LabelArray[i]:=TLabelEx.Create(Self, sl[i], aFontColor, aHiliteColor, aBackgroundColor,
                                     MenuWidth, ItemHeight, th);
      with LabelArray[i] do begin
        Tag:=i;
        AnchorSideLeft.Control:=Self;
        case i of
          0: AnchorSideTop.Control:=Self;
          else begin
            AnchorSideTop.Control:=LabelArray[Pred(i)];
            AnchorSideTop.Side:=asrBottom;
          end;
        end;
      end;
    end;
  finally
    sl.Free;
  end;
  SetInitialBounds(0, 0, MenuWidth, ItemHeight*Length(LabelArray));
  AutoSize:=True;
end;

function TVerticalMenu.GetCaptions(index: Integer): string;
begin
  if IndexIsOK(index) then
    Exit(LabelArray[index].Caption)
  else Exit('');
end;

function TVerticalMenu.GetCount: Integer;
begin
  Exit(Length(LabelArray));
end;

function TVerticalMenu.GetOnClick(index: Integer): TNotifyEvent;
begin
  if IndexIsOK(index) then
    Exit(LabelArray[index].OnClick)
  else Exit(nil);
end;

function TVerticalMenu.GetTextHeight: Integer;
var
  R: TRect;
  DC: HDC;
  DrawFlags: UINT = DT_CALCRECT or DT_NOPREFIX or DT_EXPANDTABS;
  TheText: string = 'Qg';
begin
  R:=Rect(0,0,MenuWidth,ItemHeight);
  DC:=GetDC(Parent.Handle);
  try
    DrawText(DC, PChar(TheText), Length(TheText), R, DrawFlags);
    Result:=R.Height;
  finally
    ReleaseDC(Parent.Handle, DC);
  end;
end;

function TVerticalMenu.IndexIsOK(anIndex: Integer): Boolean;
begin
  Exit( (anIndex > -1) and (anIndex < Length(LabelArray)) );
end;

procedure TVerticalMenu.SetCaptions(index: Integer; AValue: string);
begin
  if IndexIsOK(index) then
    LabelArray[index].Caption:=AValue
  else Assert(True,'TVerticalMenu.SetCaptions: invalid index '+IntToStr(index));
end;

procedure TVerticalMenu.SetOnClick(index: Integer; AValue: TNotifyEvent);
begin
  if IndexIsOK(index) then
    LabelArray[index].OnClick:=AValue
  else Assert(True,'TVerticalMenu.SetOnClick: invalid index '+IntToStr(index));
end;

{ TLabelEx }

constructor TLabelEx.Create(anOwnerParent: TCustomControl;
  const aCaption: string; aFontColor, aHiliteColor, aBackgroundColor: TColor;
  aWidth, aHeight, aTextHeight: Integer);
begin
  Assert(Assigned(anOwnerParent),'TLabelEx.Create: owner is nil');
  Assert(aCaption<>'','TLabelEx.Create: blank caption');
  inherited Create(anOwnerParent);
  Caption:=aCaption;
  Font.Color:=aFontColor;
  HiliteColor:=aHiliteColor;
  BackgroundColor:=aBackgroundColor;
  Color:=aBackgroundColor;
  Ht:=aHeight;
  Wd:=aWidth;
  TextHeight:=aTextHeight;
  Canvas.Brush.Color:=aBackgroundColor;
  Font.Quality:=fqAntialiased;
  Layout:=tlCenter;
  AutoSize:=False;
  SetInitialBounds(0, 0, aWidth, aHeight);
  Parent:=anOwnerParent;
end;

procedure TLabelEx.MouseEnter;
begin
  inherited MouseEnter;
  Repaint;
end;

procedure TLabelEx.MouseLeave;
begin
  inherited MouseLeave;
  Repaint;
end;

procedure TLabelEx.Paint;
var
  pts: array of TPoint;
  h: Integer;
begin
  h:=(Ht - TextHeight) div 2;
  case MouseEntered of
    False: begin
      Canvas.Brush.Color:=BackgroundColor;
      Canvas.FillRect(ClientRect);
      Font.Style:=[];
      Canvas.TextOut(CaptionOffset, h, Caption);
    end;
    True: begin
      Canvas.Brush.Color:=HiliteColor;
      Canvas.FillRect(ClientRect);
      Font.Style:=[fsBold];
      Canvas.TextOut(CaptionOffset, h, Caption);
      SetLength(pts, 3);
      h:=ClientHeight div 2;
      pts[0]:=Point(0, h-4);
      pts[1]:=Point(6, h);
      pts[2]:=Point(0, h+4);
      Canvas.Brush.Color:=BackgroundColor;
      Canvas.Polygon(pts);
    end;
  end;
end;

end.

